@@ -35,7 +35,7 @@
  * jl  10.02.2000 parameter P_STOPB
  */
 
-#define PARS_VAL_LEN 64
+#define PARS_VAL_LEN 128
 
 struct pars {
   /* value is first, so that (char *)mpars[0] == mpars[0].value */
@@ -162,8 +162,19 @@ enum config_type {
 /* Terminal behaviour */
 #define P_LOCALECHO	mpars[87].value
 #define P_ADDLINEFEED	mpars[88].value
+#define P_LINEWRAP      mpars[89].value  /* Line wrap */
+#define P_DISPLAYHEX    mpars[90].value  /* Do output as hex */
+#define P_ADDCARRIAGERETURN mpars[91].value
 
-#define P_ANSWERBACK    mpars[89].value  /* User defined answerback string */
+#define P_ANSWERBACK    mpars[92].value  /* User defined answerback string */
+
+#define P_RS485_EN              mpars[93].value  /* RS485 Enable */
+#define P_RS485_RTS_ON_SEND     mpars[94].value  /* RS485 Rts on send */
+#define P_RS485_RTS_AFTER_SEND  mpars[95].value  /* RS485 Rts after send */
+#define P_RS485_RX_DURING_TX    mpars[96].value  /* RS485 Rx during tx */
+#define P_RS485_TERMINATE_BUS   mpars[97].value  /* RS485 Terminate bus */
+#define P_RS485_DEL_RTS_BEF_SND mpars[98].value  /* RS485 Delay rts before send */
+#define P_RS485_DEL_RTS_AFT_SND mpars[99].value  /* RS485 Delay rts after send */
 
 /* fmg - macros struct */
 
@@ -176,5 +187,7 @@ enum config_type {
 #define P_MAC7          mmacs[6].value
 #define P_MAC8          mmacs[7].value
 #define P_MAC9          mmacs[8].value
-#define P_MAC10          mmacs[9].value
+#define P_MAC10         mmacs[9].value
+#define P_MAC11         mmacs[10].value
+#define P_MAC12         mmacs[11].value
 
