@@ -32,18 +32,28 @@
 #endif
 
 #include <stdbool.h>
+#include <assert.h>
 
 #ifdef SVR4_LOCKS
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <sys/mkdev.h>
 #endif /* SVR4_LOCKS */
 
 static jmp_buf albuf;
 
+#ifdef USE_SOCKET
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <netdb.h>
+static const char SOCKET_PREFIX_UNIX[] = "unix:";
+static const char SOCKET_PREFIX_UNIX_LEGACY[] = "unix#";
+static const char SOCKET_PREFIX_TCP[] = "tcp:";
+#endif
+
 /* Compile SCCS ID into executable. */
 const char *Version = VERSION;
 
+#ifndef SVR4_LOCKS
 /*
  * Find out name to use for lockfile when locking tty.
  */
@@ -70,6 +80,7 @@ static char *mdevlockname(char *s, char *res, int reslen)
 
   return res;
 }
+#endif
 
 static char *shortened_devpath(char *buf, int buflen, char *devpath)
 {
@@ -144,23 +155,87 @@ static void get_alrm(int dummy)
 }
 
 #ifdef USE_SOCKET
+static void term_socket_connect_unix(void)
+{
+  struct sockaddr_un sa_un;
+  sa_un.sun_family = AF_UNIX;
+  strncpy(sa_un.sun_path,
+	  dial_tty + strlen(SOCKET_PREFIX_UNIX),
+	  sizeof(sa_un.sun_path) - 1);
+  sa_un.sun_path[sizeof(sa_un.sun_path) - 1] = 0;
+
+  if ((portfd = socket(AF_UNIX, SOCK_STREAM, 0)) == -1)
+    return;
+
+  if (connect(portfd, (struct sockaddr *)&sa_un, sizeof(sa_un)) == -1)
+    term_socket_close();
+  else
+    portfd_is_connected = 1;
+}
+
+static void term_socket_connect_tcp(void)
+{
+  struct addrinfo hints;
+
+  memset(&hints, 0, sizeof(hints));
+  hints.ai_family   = AF_UNSPEC;
+  hints.ai_socktype = SOCK_STREAM;
+
+  char *s = strdup(dial_tty);
+  if (!s)
+    return;
+
+  char *s_addr = s + strlen(SOCKET_PREFIX_TCP);
+  char *s_port = strchr(s_addr, ':');
+  if (!s_port) {
+    fprintf(stderr, "No port given\n");
+    return;
+  }
+  *s_port = 0;
+  s_port++;
+
+  if (strlen(s_addr) == 0)
+    s_addr = "localhost";
+
+  struct addrinfo *result;
+  int r = getaddrinfo(s_addr, s_port, &hints, &result);
+  if (r) {
+    fprintf(stderr, "Name resolution failed: %s\n", gai_strerror(r));
+    return;
+  }
+
+  free(s);
+
+  struct addrinfo *rp;
+  for (rp = result; rp; rp = rp->ai_next) {
+    portfd = socket(rp->ai_family, rp->ai_socktype, rp->ai_protocol);
+    if (portfd == -1)
+      continue;
+
+    if (connect(portfd, rp->ai_addr, rp->ai_addrlen) != -1) {
+      portfd_is_connected = 1;
+      break;
+    }
+
+    term_socket_close();
+  }
+
+  if (rp)
+    freeaddrinfo(result);
+}
+
 /*
  * If portfd is a socket, we try to (re)connect
  */
 void term_socket_connect(void)
 {
-
-  if (!portfd_is_socket || portfd_is_connected)
+  if (portfd_is_socket == Socket_type_no_socket || portfd_is_connected)
     return;
 
-  if ((portfd = socket(AF_UNIX, SOCK_STREAM, 0)) == -1)
-    return;
-
-  if (connect(portfd, (struct sockaddr *)&portfd_sock_addr,
-              sizeof(portfd_sock_addr)) == -1)
-    term_socket_close();
-  else
-    portfd_is_connected = 1;
+  if (portfd_is_socket == Socket_type_unix)
+    term_socket_connect_unix();
+  else if (portfd_is_socket == Socket_type_tcp)
+    term_socket_connect_tcp();
 }
 
 /*
@@ -188,17 +263,23 @@ int open_term(int doinit, int show_win_on_error, int no_msgs)
   } buf;
   int fd, n = 0;
   int pid;
+#ifdef USE_SOCKET
 #ifdef HAVE_ERRNO_H
   int s_errno;
 #endif
+#endif /* USE_SOCKET */
 
 #ifdef USE_SOCKET
-#define SOCKET_PREFIX "unix#"
-    portfd_is_socket = portfd_is_connected = 0;
-    if (strncmp(dial_tty, SOCKET_PREFIX, strlen(SOCKET_PREFIX)) == 0) {
-      portfd_is_socket = 1;
-    }
-#endif
+  portfd_is_socket = portfd_is_connected = 0;
+  int ulen = strlen(SOCKET_PREFIX_UNIX);
+  assert(ulen == strlen(SOCKET_PREFIX_UNIX_LEGACY));
+  if (!strncmp(dial_tty, SOCKET_PREFIX_UNIX, ulen))
+    portfd_is_socket = Socket_type_unix;
+  else if (!strncmp(dial_tty, SOCKET_PREFIX_UNIX_LEGACY, ulen))
+    portfd_is_socket = Socket_type_unix;
+  else if (!strncmp(dial_tty, SOCKET_PREFIX_TCP, strlen(SOCKET_PREFIX_TCP)))
+    portfd_is_socket = Socket_type_tcp;
+#endif /* USE_SOCKET */
 
   if (portfd_is_socket)
     goto nolock;
@@ -216,7 +297,8 @@ int open_term(int doinit, int show_win_on_error, int no_msgs)
 #else /* SVR4_LOCKS */
     snprintf(lockfile, sizeof(lockfile),
                        "%s/LCK..%s",
-                       P_LOCK, mdevlockname(dial_tty, buf.bytes, sizeof(buf.bytes)));
+                       P_LOCK, mdevlockname(dial_tty, buf.bytes,
+                                            sizeof(buf.bytes)));
 #endif /* SVR4_LOCKS */
 
   }
@@ -275,11 +357,6 @@ nolock:
     alarm(20);
 #ifdef USE_SOCKET
     if (portfd_is_socket) {
-      portfd_sock_addr.sun_family = AF_UNIX;
-      strncpy(portfd_sock_addr.sun_path,
-              dial_tty + strlen(SOCKET_PREFIX),
-              sizeof(portfd_sock_addr.sun_path)-1);
-      portfd_sock_addr.sun_path[sizeof(portfd_sock_addr.sun_path)-1] = 0;
       term_socket_connect();
     }
 #endif /* USE_SOCKET */
@@ -302,12 +379,15 @@ nolock:
       port_init();
     }
   }
+#ifdef USE_SOCKET
 #ifdef HAVE_ERRNO_H
   s_errno = errno;
 #endif
+#endif /* USE_SOCKET */
   alarm(0);
   signal(SIGALRM, SIG_IGN);
-  if (portfd < 0 && !portfd_is_socket) {
+#ifdef USE_SOCKET
+  if (portfd < 0 && portfd_is_socket == Socket_type_no_socket) {
     if (!no_msgs) {
       if (doinit > 0) {
 	if (stdwin)
@@ -329,6 +409,9 @@ nolock:
     lockfile_remove();
     return -1;
   }
+#else
+  (void)show_win_on_error;
+#endif /* USE_SOCKET */
 
   /* Set CLOCAL mode */
   m_nohang(portfd);
@@ -481,8 +564,9 @@ static int get_device_status(int fd)
 static char status_message[80];
 static int  status_display_msg_until;
 static int  status_message_showing;
+static char *current_status_line;
 
-static const char default_statusline_format[] = "%H for help | %b | %C | Minicom %V | %T | %t | %D";
+static const char default_statusline_format[] = N_("%H for help | %b | %C | Minicom %V | %T | %t | %D");
 
 static const char *statusline_format = default_statusline_format;
 
@@ -521,9 +605,13 @@ static void show_status_fmt(const char *fmt)
               bufi += snprintf(buf + bufi, COLS - bufi, "%s", VERSION);
               break;
             case 'b':
-              if (portfd_is_socket)
+#ifdef USE_SOCKET
+              if (portfd_is_socket == Socket_type_unix)
                 bufi += snprintf(buf + bufi, COLS - bufi, "unix-socket");
+	      else if (portfd_is_socket == Socket_type_tcp)
+                bufi += snprintf(buf + bufi, COLS - bufi, "TCP");
               else
+#endif /* USE_SOCKET */
                 {
                   if (P_SHOWSPD[0] == 'l')
                     bufi += snprintf(buf + bufi, COLS - bufi, "%6ld", linespd);
@@ -548,7 +636,7 @@ static void show_status_fmt(const char *fmt)
               bufi += snprintf(buf + bufi, COLS - bufi, cursormode == NORMAL ? "NOR" : "APP");
               break;
 
-            case 't':
+	    case 't':
               if (online < 0)
                 bufi += snprintf(buf + bufi, COLS - bufi, "%s",
                                  P_HASDCD[0] == 'Y' ? _("Offline") : _("OFFLINE"));
@@ -562,7 +650,7 @@ static void show_status_fmt(const char *fmt)
                 {
                   char b[COLS - bufi];
                   bufi += snprintf(buf + bufi, COLS - bufi, "%s",
-                                   shortened_devpath(b, sizeof(b), P_PORT));
+                                   shortened_devpath(b, sizeof(b), dial_tty));
                 }
               break;
 
@@ -586,16 +674,22 @@ static void show_status_fmt(const char *fmt)
     memset(buf + bufi, ' ', COLS - bufi);
   buf[COLS - 1] = 0;
 
-  st->direct = 0;
-  mc_wlocate(st, 0, 0);
-  mc_wprintf(st, "%s", buf);
-  mc_wredraw(st, 1);
-  ret_csr();
+  if (size_changed || !current_status_line || strcmp(buf, current_status_line))
+    {
+      st->direct = 0;
+      mc_wlocate(st, 0, 0);
+      mc_wprintf(st, "%s", buf);
+      mc_wredraw(st, 1);
+      ret_csr();
+      current_status_line = realloc(current_status_line, COLS);
+      assert(current_status_line);
+      strcpy(current_status_line, buf);
+    }
 }
 
 void show_status()
 {
-  show_status_fmt(statusline_format);
+  show_status_fmt(gettext(statusline_format));
 }
 
 time_t old_online = -2;
@@ -620,9 +714,6 @@ static void update_status_time(void)
         show_status_fmt("%$");
     }
 
-  if (old_online == online || online <= (old_online + 59))
-    return;
-
   if (P_LOGCONN[0] == 'Y' && old_online >= 0 && online < 0)
     do_log(_("Gone offline (%ld:%02ld:%02ld)"),
            old_online / 3600, (old_online / 60) % 60, old_online % 60);
@@ -656,7 +747,7 @@ void status_set_display(const char *text, int duration_s)
 void timer_update(void)
 {
   static time_t t1, start;
-  int dcd_support = P_HASDCD[0] == 'Y';
+  int dcd_support = portfd_is_socket || P_HASDCD[0] == 'Y';
 
   /* See if we're online. */
   if ((!dcd_support && bogus_dcd)
@@ -772,7 +863,6 @@ dirty_goto:
   while (1) {
     /* See if window size changed */
     if (size_changed) {
-      size_changed = 0;
       wrapln = us->wrap;
       /* I got the resize code going again! Yeah! */
       mc_wclose(us, 0);
@@ -786,12 +876,13 @@ dirty_goto:
       /* Set the terminal modes */
       setcbreak(2); /* Raw, no echo */
       init_emul(terminal, 0);
+      size_changed = 0;
     }
     /* Update the timer. */
     timer_update();
 
     /* check if device is ok, if not, try to open it */
-    if (!get_device_status(portfd_connected)) {
+    if (!get_device_status(portfd_connected())) {
       /* Ok, it's gone, most probably someone unplugged the USB-serial, we
        * need to free the FD so that a replug can get the same device
        * filename, open it again and be back */
@@ -811,8 +902,7 @@ dirty_goto:
     }
 
     /* Check for I/O or timer. */
-    x = check_io(portfd_connected, 0, 1000,
-                 buf + buf_offset, sizeof(buf) - buf_offset, &blen);
+    x = check_io_frontend(buf + buf_offset, sizeof(buf) - buf_offset, &blen);
     blen += buf_offset;
     buf_offset = 0;
 
@@ -847,25 +937,34 @@ dirty_goto:
         ptr = buf;
       }
 
-      while (blen-- > 0) {
+      while (blen > 0) {
+	int c = *ptr;
         /* Auto zmodem detect */
         if (zauto) {
-          if (zsig[zpos] == *ptr)
+          if (zsig[zpos] == c)
             zpos++;
           else
             zpos = 0;
         }
         if (P_PARITY[0] == 'M' || P_PARITY[0] == 'S')
-          *ptr &= 0x7f;
+          c &= 0x7f;
         if (display_hex) {
-          unsigned char c = *ptr++;
-          unsigned char u = c >> 4;
-          c &= 0xf;
-          vt_out(u > 9 ? 'a' + (u - 10) : '0' + u);
-          vt_out(c > 9 ? 'a' + (c - 10) : '0' + c);
-          vt_out(' ');
-        } else
-          vt_out(*ptr++);
+          unsigned char l = c;
+          unsigned char u = l >> 4;
+          l &= 0xf;
+          vt_out(u > 9 ? 'a' + (u - 10) : '0' + u, 0);
+          vt_out(l > 9 ? 'a' + (l - 10) : '0' + l, 0);
+          vt_out(' ', 0);
+
+          --blen;
+          ++ptr;
+        } else {
+          wchar_t wc;
+          size_t len = one_mbtowc(&wc, ptr, blen);
+          vt_out(c, wc);
+          blen -= len;
+          ptr += len;
+        }
         if (zauto && zsig[zpos] == 0) {
           dirflush = 1;
           keyboard(KSTOP, 0);
@@ -924,7 +1023,8 @@ dirty_goto:
       }
 
       /* No, just a key to be sent. */
-      if (c >= K_F1 && c <= K_F10 && P_MACENAB[0] == 'Y') {
+      if (((c >= K_F1 && c <= K_F10) || c == K_F11 || c == K_F12)
+	  && P_MACENAB[0] == 'Y') {
         s = "";
         switch(c) {
           case K_F1: s = P_MAC1; break;
@@ -937,10 +1037,13 @@ dirty_goto:
           case K_F8: s = P_MAC8; break;
           case K_F9: s = P_MAC9; break;
           case K_F10: s = P_MAC10; break;
+          case K_F11: s = P_MAC11; break;
+          case K_F12: s = P_MAC12; break;
         }
-        if (*s)
-          mputs(s, 1);
-        else
+        if (*s) {
+	  while (*s)
+	    vt_send(*s++);
+	} else
           vt_send(c);
       } else
         vt_send(c);
