@@ -102,19 +102,8 @@ static int _mv_standout = 0;
 static ELM oldc;
 static int sflag = 0;
 
-/*
- * Smooth is only defined for slow machines running Minicom.
- * With this defined, Minicom will buffer only per-line
- * and the output will look much less 'jerky'. (I hope :-)
- */
-#ifdef SMOOTH
-static WIN *curwin = NULL;
-WIN *us;
-#endif
-
 int useattr = 1;
 int dirflush = 1;
-int LINES, COLS;
 int usecolor = 0;
 WIN *stdwin;
 /*
@@ -128,7 +117,7 @@ int screen_iso = 0;
 int w_init = 0;
 int use_status = 0; /* Turned on in main() */
 
-/* Standard vt100 map (ac cpability) */
+/* Standard vt100 map (ac capability) */
 static const char *def_ac = "+\273,\253aaffggjjkkllmmnnooqqssttuuvvwwxx";
 
 #ifdef DEBUG
@@ -181,10 +170,6 @@ static int outchar(int c)
   *_bufpos++ = c;
   if (_bufpos >= _buffend)
     mc_wflush();
-#if defined(SMOOTH)
-  if (curwin == us && (c == '\n' || c == '\r'))
-    mc_wflush();
-#endif
   return 0;
 }
 
@@ -380,15 +365,15 @@ static void _write(wchar_t c, int doit, int x, int y, char attr, char color)
         _setattr(attr, color);
       }
       x0 = x; y0 = y; attr0 = attr; color0 = color; c0 = c;
-      if (using_iconv() || (attr & XA_ALTCHARSET) != 0)
+      if ((attr & XA_ALTCHARSET) != 0)
         outchar((char)c);
       else {
         char buf[MB_LEN_MAX];
         size_t i, len;
 
         len = one_wctomb(buf, c);
-        for (i = 0; i < (size_t)len; i++)
-        outchar(buf[i]);
+        for (i = 0; i < len; i++)
+          outchar(buf[i]);
       }
 
       curx++;
@@ -480,9 +465,6 @@ WIN *mc_wopen(int x1, int y1, int x2, int y2, int border, int attr,
   int color;
   int offs;
   int xattr;
-#ifdef SMOOTH
-  curwin = NULL;
-#endif
 
   if ((w = malloc(sizeof(WIN))) == NULL)
     return w;
@@ -611,10 +593,6 @@ void mc_wclose(WIN *win, int replace)
   ELM *e;
   int x, y;
 
-#ifdef SMOOTH
-  curwin = NULL;
-#endif
-
   if (!win)
     return;
 
@@ -693,10 +671,6 @@ void mc_wreturn(void)
   int x, y;
   ELM *e;
 
-#ifdef SMOOTH
-  curwin = NULL;
-#endif
-
   curattr = -1;
   curcolor = -1;
 
@@ -773,9 +747,6 @@ static int _wclreol(WIN *w)
   int doit = 1;
   int y;
 
-#ifdef SMOOTH
-  curwin = w;
-#endif
   y = w->cury + w->y1;
 
   if (w->direct && (w->x2 == COLS - 1) && CE) {
@@ -802,10 +773,6 @@ void mc_wscroll(WIN *win, int dir)
   int ocurx, fs = 0, len;
   int phys_scr = 0;
 
-#ifdef SMOOTH
-  curwin = win;
-#endif
-
   /*
    * If the window *is* the physical screen, we can scroll very simple.
    * This improves performance on slow screens (eg ATARI ST) dramatically.
@@ -898,7 +865,7 @@ void mc_wscroll(WIN *win, int dir)
     /* Copy line from screen to history buffer */
     memcpy((char *)f, (char *)e, win->xs * sizeof(ELM));
 
-    /* Postion the next line in the history buffer */
+    /* Position the next line in the history buffer */
     win->histline++;
     if (win->histline >= win->histlines)
       win->histline = 0;
@@ -1001,10 +968,6 @@ void mc_wputc(WIN *win, wchar_t c)
 {
   int mv = 0;
 
-#ifdef SMOOTH
-  curwin = win;
-#endif
-
   switch(c) {
     case '\r':
       win->curx = 0;
@@ -1027,7 +990,7 @@ void mc_wputc(WIN *win, wchar_t c)
     case '\n':
       if (win->autocr)
         win->curx = 0;
-      /*FALLTHRU*/
+      /* FALLTHRU */
     default:
       /* See if we need to scroll/move. (vt100 behaviour!) */
       if (c == '\n' || (win->curx >= win->xs && win->wrap)) {
@@ -1142,7 +1105,7 @@ void mc_wputs(WIN *win, const char *s)
 
 /*
  * Print a formatted string in a window.
- * Should return stringlength - but who cares.
+ * Should return string length - but who cares.
  */
 int mc_wprintf(WIN *win, const char *fmt, ...)
 {
@@ -1188,19 +1151,15 @@ void mc_wtitle(WIN *w, int pos, const char *s)
 {
   int x = 0;
 
-#ifdef SMOOTH
-  curwin = NULL;
-#endif
-
   if (w->border == BNONE)
     return;
 
   if (pos == TLEFT)
     x = w->x1;
   if (pos == TRIGHT)
-    x = w->x2 - mbslen(s) - 1;
+    x = w->x2 - mbswidth(s) - 1;
   if (pos == TMID)
-    x = w->x1 + (w->xs - mbslen(s)) / 2 - 1;
+    x = w->x1 + (w->xs - mbswidth(s)) / 2 - 1;
   if (x < w->x1)
     x = w->x1;
 
@@ -1233,10 +1192,6 @@ void mc_wcurbar(WIN *w, int y, int attr)
   ELM *e;
   int x;
 
-#ifdef SMOOTH
-  curwin = w;
-#endif
-
   y += w->y1;
 
   e = gmap + y * COLS + w->x1;
@@ -1282,12 +1237,12 @@ int mc_wselect(int x, int y, const char *const *choices,
 
   while (*a != NULL) {
     count++;
-    if (mbslen(_(*a)) > len)
-      len = mbslen(_(*a));
+    if (mbswidth(_(*a)) > len)
+      len = mbswidth(_(*a));
     a++;
   }
-  if (title != NULL && mbslen(title) + 2 > len)
-    len = mbslen(title) + 2;
+  if (title != NULL && mbswidth(title) + 2 > len)
+    len = mbswidth(title) + 2;
   if (attr & XA_REVERSE) {
     high_on = attr & ~XA_REVERSE;
     high_off = attr;
@@ -1360,10 +1315,6 @@ void mc_wclrch(WIN *w, int n)
 {
   int x, y, x_end;
 
-#ifdef SMOOTH
-  curwin = w;
-#endif
-
   y = w->cury + w->y1;
   x = x_end = w->curx + w->x1;
   x_end += n - 1;
@@ -1412,10 +1363,6 @@ void mc_wclrbol(WIN *w)
 {
   int x, y, n;
 
-#ifdef SMOOTH
-  curwin = w;
-#endif
-
   y = w->cury + w->y1;
 
   if (w->direct)
@@ -1543,6 +1490,17 @@ void mc_winclr(WIN *w)
     mc_wflush();
 }
 
+void mc_clear_window_simple(WIN *w)
+{
+  int x = 0, y = 0;
+  _colson(us->color);
+  _gotoxy(0, 0);
+  for (; y <= w->ys; ++y)
+    for (; x <= w->xs; ++x)
+      _write(' ', 1, x, y, XA_NORMAL, us->color);
+  _gotoxy(0, 0);
+}
+
 /* ==== Insert / Delete functions ==== */
 
 void mc_winsline(WIN *w)
@@ -1601,10 +1559,6 @@ void mc_winschar2(WIN *w, wchar_t c, int move)
   int len, odir;
   int oldx;
 
-#ifdef SMOOTH
-  curwin = w;
-#endif
-
   /* See if we need to insert. */
   if (c == 0 || wcschr(L"\r\n\t\b\007", c) || w->curx >= w->xs - 1) {
     mc_wputc(w, c);
@@ -1667,10 +1621,6 @@ void mc_wdelchar(WIN *w)
   int doit = 1;
   ELM *e;
 
-#ifdef SMOOTH
-  curwin = w;
-#endif
-
   x = w->x1 + w->curx;
   y = w->y1 + w->cury;
 
@@ -1693,7 +1643,7 @@ void mc_wdelchar(WIN *w)
 
 /* ============= Support: edit a line on the screen. ============ */
 
-/* Redraw the line we are editting. */
+/* Redraw the line we are editing. */
 static void lredraw(WIN *w, int x, int y, wchar_t *s, int len)
 {
   int i, f;
@@ -1737,7 +1687,7 @@ int mc_wgetwcs(WIN *w, wchar_t *s, int linelen, int maxlen)
   /* We assume the line has already been drawn on the screen. */
   if ((idx = wcslen(s)) > linelen)
     idx = linelen;
-  wcsncpy(buf, s, sizeof(buf) / sizeof(*buf));
+  wcsncpy(buf, s, ARRAY_SIZE(buf));
   mc_wlocate(w, x + idx, y);
   dirflush = 0;
   mc_wflush();
@@ -1776,7 +1726,7 @@ int mc_wgetwcs(WIN *w, wchar_t *s, int linelen, int maxlen)
       case K_END: /* End of line. */
         idx = wcslen(buf);
         r = 0;
-        while (idx - offs > linelen) {
+        while (idx - offs - 3 > linelen) {
           r = 1;
           offs += 4;
         }
@@ -1800,7 +1750,7 @@ int mc_wgetwcs(WIN *w, wchar_t *s, int linelen, int maxlen)
           mc_wflush();
           break;
         }
-        /*FALLTHRU*/
+        /* FALLTHRU */
       case K_DEL: /* Delete character under cursor. */
         if (buf[idx] == 0)
           break;
@@ -1872,7 +1822,7 @@ int mc_wgets(WIN *w, char *s, int linelen, int maxlen)
   char *sptr;
 
   sptr = s;
-  for (i = 0; *sptr != 0 && i < sizeof (buf) - 1; i++)
+  for (i = 0; *sptr != 0 && i < ARRAY_SIZE(buf) - 1; i++)
     sptr += one_mbtowc(buf + i, sptr, MB_LEN_MAX);
   buf[i] = 0;
   st = mc_wgetwcs(w, buf, linelen, maxlen);
@@ -2103,7 +2053,7 @@ int win_init(int fg, int bg, int attr)
 
   /* Memory for global map */
   if ((gmap = malloc(sizeof(ELM) * (LINES + 1) * COLS)) == NULL) {
-    fprintf(stderr, "Not enough memory\n");
+    fprintf(stderr, _("Not enough memory\n"));
     return -1;
   };
   _buffend = _bufstart + BUFFERSIZE;
